@@ -32,10 +32,16 @@ G_BEGIN_DECLS
 
 void _gdm_user_update           (GdmUser             *user,
                                  const struct passwd *pwent);
-void _gdm_user_set_is_logged_in (GdmUser             *user,
-                                 gboolean             is_logged_in);
+void _gdm_user_add_session      (GdmUser             *user,
+                                 const char          *session_id);
+void _gdm_user_remove_session   (GdmUser             *user,
+                                 const char          *session_id);
+
 void _gdm_user_icon_changed     (GdmUser             *user);
 
+void _gdm_user_show_full_display_name (GdmUser   *user);
+void _gdm_user_show_short_display_name  (GdmUser   *user);
+
 G_END_DECLS
 
 #endif /* !__GDM_USER_PRIVATE__ */
